###################################
# SimpleXmlSearcher.py 
# 
# A simple sax implementation to search xml for tags and get their 
# text content if they are present. See search method for use detail.
#
# Author   : Peter Simmons
# Copyright: BCMP parters 2000
# License  : TBA
#
###################################

from xml.sax import saxutils
from xml.sax import make_parser
from xml.sax.handler import feature_namespaces
from StringIO import StringIO
import Globals
import AccessControl
import OFS

#creation methods

def manage_addSimpleXmlSearcher(self, id, title, REQUEST=None):
    """Create a SimpleXmlSearcher and install it in its parent Folder.
    'self' is bound to the parent folder"""
    sxs = SimpleXmlSearcher(id, title)
    self._setObject(id, sxs)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

manage_addSimpleXmlSearcherForm = Globals.HTMLFile('dtml/addSimpleXMLSearcher', globals())

use_notice = Globals.HTMLFile('dtml/use_notice', globals())

#class definition

class SimpleXmlSearcher(saxutils.DefaultHandler,
			OFS.SimpleItem.SimpleItem):
    """Xml Parser that searches for vlaues and returns the text content of the node"""
    
    #zope product stuff
    manage_options=(
     	    {'label':'Manage', 'action':'index_html'},       
	    {'label':'Ownership', 'action': 'manage_owner'},
            ) + AccessControl.Role.RoleManager.manage_options
    
    meta_type = 'Simple Xml Searcher'
    
    __ac_permissions__=(
            ('View management screens', ('manage_main', 'index_html',)),
            ('Search Xml', ('search',)),
            )

    #some dtml files to use in zope, they go with the version of the 
    #product so they aren't editable at the zope level
    use_notice = Globals.HTMLFile('dtml/use_notice', globals())
    index_html = Globals.HTMLFile('dtml/index_html', globals())
    manage_main = index_html
    
    def __init__(self, id, title):
        """Initialises the object. Sets  all the  variables to their empty values.
              search for is the method to call to do the searching"""
        self.tags = []
        self._values= Globals.PersistentMapping()
        self.isTagValue = 0
        self.current_tag = ''
        self.id = id
        self.title = title

    #xml parser methods. Doc comment is just comment so not accessible from
    #zope

    def startElement(self, name, attrs):
        #gets called whenever a start tag is parsed
        #if the tag is one we are looking for note it
        if name in self.tags:
            self.isTagValue = 1
            self.current_tag = name
                                            
    def endElement(self, name):
        #called when an end tag is parsed
        #NB: <tag/> causes startElement then endElement straight away
        isTagValue = 0
    
    def characters(self, ch):
        #called when characters (text content) is parsed
        #if we are currently within a tag we are looking for
        #get its value and put it in self._values
        if self.isTagValue:
            if self._values.has_key(self.current_tag):
                obj = self._values[self.current_tag]
                obj = obj + ch
            else:
                self._values[self.current_tag] = ch

    #zope interface method, call this to search an xml document

    def search(self, xml='', tags=[], REQUEST=None):
        """Get the values of the text inside (if there is a tag) of 
	   each element of tags and puts it in the REQUEST as a tag: value pair.
	   The parameters xml and/or tags can be passed in via the REQUEST with
	   direct passing overriding values in REQUEST
	   xml is a string of xml e.g. render(dtml_method_containing_xml)
	"""
	if len(tags) <= 0:
	    if REQUEST.has_key('tags'):
		tags = REQUEST['tags']
	    else:
		raise ValueError('tags')

	if len(xml) <= 0:
	    if REQUEST.has_key('xml'):
		xml = REQUEST['xml']
	    else:
		raise ValueError('xml')
	
        if len(tags) > 0 and len(xml) > 0:
	    self.tags = tags
	    self._values = Globals.PersistentMapping()

            parser = make_parser()
            parser.setFeature(feature_namespaces, 0)
            parser.setContentHandler(self)
            
            parser.parse(StringIO(xml))

	    REQUEST.set('tags', self.tags)

	    for i in self._values.keys():
		REQUEST.set(i, self._values[i])

	    return REQUEST

#register permissions with zope
Globals.default__class_init__(SimpleXmlSearcher)